"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var PairingHeap = require("./PairingHeap");
var PriorityQueueIterator_1 = require("./PriorityQueueIterator");
exports.DEFAULT_COMPARATOR = function (a, b) {
    if (a < b) {
        return -1;
    }
    else if (a > b) {
        return 1;
    }
    else {
        return 0;
    }
};
exports.NIL = { heap: undefined, size: 0 };
function make(arraylike, comparator) {
    if (comparator === void 0) { comparator = exports.DEFAULT_COMPARATOR; }
    var heap = undefined;
    var size = 0;
    if (arraylike) {
        for (var _i = 0, arraylike_1 = arraylike; _i < arraylike_1.length; _i++) {
            var _a = arraylike_1[_i], element = _a[0], priority = _a[1];
            heap = PairingHeap.insert(heap, element, priority, comparator);
            size++;
        }
    }
    return { heap: heap, size: size };
}
exports.make = make;
function isEmpty(_a) {
    var size = _a.size;
    return size === 0;
}
exports.isEmpty = isEmpty;
function clear(_) {
    return exports.NIL;
}
exports.clear = clear;
function first(_a) {
    var heap = _a.heap;
    return !heap ? undefined : heap.entry;
}
exports.first = first;
function firstElement(_a) {
    var heap = _a.heap;
    return !heap ? undefined : heap.entry[0];
}
exports.firstElement = firstElement;
function firstPriority(_a) {
    var heap = _a.heap;
    return !heap ? undefined : heap.entry[1];
}
exports.firstPriority = firstPriority;
function enqueue(_a, element, priority, comparator) {
    var heap = _a.heap, size = _a.size;
    if (comparator === void 0) { comparator = exports.DEFAULT_COMPARATOR; }
    return { heap: PairingHeap.insert(heap, element, priority, comparator), size: size + 1 };
}
exports.enqueue = enqueue;
function dequeue(queue, comparator) {
    if (comparator === void 0) { comparator = exports.DEFAULT_COMPARATOR; }
    var heap = queue.heap, size = queue.size;
    if (!heap) {
        return queue;
    }
    else {
        return { heap: PairingHeap.deleteMin(heap, comparator), size: size - 1 };
    }
}
exports.dequeue = dequeue;
function merge(queue, otherQueue, comparator) {
    if (comparator === void 0) { comparator = exports.DEFAULT_COMPARATOR; }
    if (queue == otherQueue) {
        return queue;
    }
    else {
        return { heap: PairingHeap.merge(queue.heap, otherQueue.heap, comparator), size: queue.size + otherQueue.size };
    }
}
exports.merge = merge;
function has(queue, element) {
    var iterator = new PriorityQueueIterator_1.PriorityQueueIterator(queue);
    while (iterator.hasNext()) {
        if (iterator.getNext()[0] === element) {
            return true;
        }
    }
    return false;
}
exports.has = has;
function forEach(queue, callbackFn, thisArg, callbackArg) {
    if (callbackArg === void 0) { callbackArg = queue; }
    var iterator = new PriorityQueueIterator_1.PriorityQueueIterator(queue);
    while (iterator.hasNext()) {
        var _a = iterator.getNext(), element = _a[0], priority = _a[1];
        callbackFn.call(thisArg, element, priority, callbackArg);
    }
}
exports.forEach = forEach;
function iterator(queue) {
    return new PriorityQueueIterator_1.PriorityQueueIterator(queue);
}
exports.iterator = iterator;
var PriorityQueue = (function () {
    function PriorityQueue(arraylike, comparator) {
        if (comparator === void 0) { comparator = exports.DEFAULT_COMPARATOR; }
        this.comparator = comparator;
        this.queue = make(arraylike, comparator);
    }
    Object.defineProperty(PriorityQueue.prototype, "size", {
        get: function () {
            return this.queue.size;
        },
        enumerable: true,
        configurable: true
    });
    PriorityQueue.ofData = function (queue, comparator) {
        if (comparator === void 0) { comparator = exports.DEFAULT_COMPARATOR; }
        var q = new PriorityQueue(undefined, comparator);
        q.queue = queue;
        return q;
    };
    PriorityQueue.prototype.isEmpty = function () {
        return isEmpty(this.queue);
    };
    PriorityQueue.prototype.clear = function () {
        if (this.isEmpty()) {
            return this;
        }
        else {
            return new PriorityQueue(undefined, this.comparator);
        }
    };
    PriorityQueue.prototype.first = function () {
        return first(this.queue);
    };
    PriorityQueue.prototype.firstElement = function () {
        return firstElement(this.queue);
    };
    PriorityQueue.prototype.firstPriority = function () {
        return firstPriority(this.queue);
    };
    PriorityQueue.prototype.enqueue = function (element, priority) {
        return PriorityQueue.ofData(enqueue(this.queue, element, priority, this.comparator), this.comparator);
    };
    PriorityQueue.prototype.dequeue = function () {
        if (this.isEmpty())
            return this;
        return PriorityQueue.ofData(dequeue(this.queue, this.comparator), this.comparator);
    };
    PriorityQueue.prototype.merge = function (queue) {
        if (this === queue)
            return this;
        return PriorityQueue.ofData(merge(this.queue, queue.queue), this.comparator);
    };
    PriorityQueue.prototype.has = function (element) {
        return has(this.queue, element);
    };
    PriorityQueue.prototype.forEach = function (callbackFn, thisArg) {
        forEach(this.queue, callbackFn, thisArg, this);
    };
    PriorityQueue.prototype.iterator = function () {
        return iterator(this.queue);
    };
    PriorityQueue.prototype[Symbol.iterator] = function () {
        return this.iterator();
    };
    return PriorityQueue;
}());
exports.PriorityQueue = PriorityQueue;
//# sourceMappingURL=PriorityQueue.js.map